/*
*	OEM_UART.C
*
*   OEM UART related functions.
*
*	Author: Hugo Lin - Asus
*	email: Hugo_Lin@asus.com
*
*/

#include "hal_defs.h"
#include "usb_firmware_library_headers.h"
#include "hal_board.h"
#include "hal_button.h"
#include "hal_int.h"
#include "hal_led.h"
#include "hal_mcu.h"
#include "usb_hid.h"
#include "usb_hid_cert.h"

#include "hal_rf_proprietary.h"
#include "prop_regs.h"
#include "OEM_uart.h"
#include "ioCC254x_bitdef.h"
#include "dma.h"

// Variable for UART packet monitoring.
//static uint8 __xdata uartPktReceived = 0;
static uint8 __xdata uartPktTransceived = 0;

#define UART_TX_BUFFER_LENGTH       240
uint8_t __xdata uart_tx_buffer[UART_TX_BUFFER_LENGTH] = {0};
uint8_t __xdata uart_tx_head, uart_tx_tail;

// Baudrate = 115.2 kbps (U0BAUD.BAUD_M = 216, U0GCR.BAUD_E = 11), given 32 MHz system clock.
#define UART_BAUD_M  216
#define UART_BAUD_E  11

// DMA descriptor for UART RX/TX.
static DMA_DESC __xdata uartDmaRxTxCh[2];

// Prototype for local functions.
void uart0StartTxDmaChan( DMA_DESC *uartDmaTxDescr,
                          uint8 uartDmaTxChan,
                          uint8 *uartTxBuf,
                          uint16 uartTxBufSize);

void uart_fifo_init(void)
{
    uart_tx_head = 0;
    uart_tx_tail = 0;
}


void oem_uart_init(void)
{
    // Initialise bitrate = 57.6 kbps.
    U0BAUD = UART_BAUD_M;
    U0GCR = (U0GCR&~U0GCR_BAUD_E) | UART_BAUD_E;

    // Initialise UART protocol (start/stop bit, data bits, parity, etc.):
    // USART mode = UART (U0CSR.MODE = 1).
    U0CSR |= U0CSR_MODE;

    // Start bit level = low => Idle level = high  (U0UCR.START = 0).
    U0UCR &= ~U0UCR_START;

    // Stop bit level = high (U0UCR.STOP = 1).
    U0UCR |= U0UCR_STOP;

    // Number of stop bits = 1 (U0UCR.SPB = 0).
    U0UCR &= ~U0UCR_SPB;

    // Parity = disabled (U0UCR.PARITY = 0).
    U0UCR &= ~U0UCR_PARITY;

    // 9-bit data enable = 8 bits transfer (U0UCR.BIT9 = 0).
    U0UCR &= ~U0UCR_BIT9;

    // Level of bit 9 = 0 (U0UCR.D9 = 0), used when U0UCR.BIT9 = 1.
    // Level of bit 9 = 1 (U0UCR.D9 = 1), used when U0UCR.BIT9 = 1.
    // Parity = Even (U0UCR.D9 = 0), used when U0UCR.PARITY = 1.
    // Parity = Odd (U0UCR.D9 = 1), used when U0UCR.PARITY = 1.
    U0UCR &= ~U0UCR_D9;

    // Flow control = disabled (U0UCR.FLOW = 0).
    U0UCR &= ~U0UCR_FLOW;

    // Bit order = LSB first (U0GCR.ORDER = 0).
    U0GCR &= ~U0GCR_ORDER;
}

void uart_send_fifo(uint8_t buf)
{
    int diff;
    diff = (int)uart_tx_head - (int)uart_tx_tail;
    if (diff == -1 || diff >= (UART_TX_BUFFER_LENGTH -1)) {
        return; /* data over flow */
    }
    uart_tx_buffer[uart_tx_head] = buf;
    uart_tx_head++;
    if (uart_tx_head >= UART_TX_BUFFER_LENGTH) uart_tx_head = 0;
}

void uart_write_buffer_to_fifo(void)
{
    int diff;
	if (uart_tx_head > uart_tx_tail) {
		diff = uart_tx_head - uart_tx_tail;
	} else {
		diff = UART_TX_BUFFER_LENGTH - uart_tx_tail;
	}
	uartPktTransceived = 1;
	uart0StartTxDmaChan(&uartDmaRxTxCh[1], 1, uart_tx_buffer + uart_tx_tail, diff);
	uart_tx_tail += diff;
	if (uart_tx_tail >= UART_TX_BUFFER_LENGTH) uart_tx_tail = 0;

}

void check_uart_need_start(void)
{
	if (uart_tx_head != uart_tx_tail && !uartPktTransceived) {
		uart_write_buffer_to_fifo();
	}
}

/***********************************************************************************
* LOCAL FUNCTIONS
*/

#if 1
/***********************************************************************************
* @fn          uart0StartTxDmaChan
*
* @brief       Function which sets up a DMA channel for UART0 TX.
*
* @param       DMA_DESC *uartDmaTxDescr - pointer to DMA descriptor for UART TX
*              uint8 uartDmaTxChan - DMA channel number for UART TX
*              uint8* uartTxBuf - pointer to allocated UART TX buffer
*              uint16 uartTxBufSize - size of allocated UART TX buffer
*
* @return      void
*/
void uart0StartTxDmaChan( DMA_DESC *uartDmaTxDescr,
                          uint8 uartDmaTxChan,
                          uint8 *uartTxBuf,
                          uint16 uartTxBufSize)
{

    // Set source/destination pointer (UART TX buffer address) for UART TX DMA channel,
    // and total number of DMA word transfer (according to UART TX buffer size).
    uartDmaTxDescr->SRCADDRH   = (uint16)(uartTxBuf + 1) >> 8;      // +1 since we're sending the first packet manually later on,
    uartDmaTxDescr->SRCADDRL   = (uint16)(uartTxBuf + 1);           // to start the DMA
    uartDmaTxDescr->DESTADDRH  = ((uint16)(&X_U0DBUF) >> 8) & 0x00FF;
    uartDmaTxDescr->DESTADDRL  = (uint16)(&X_U0DBUF) & 0x00FF;
    uartDmaTxDescr->LENH       = ((uartTxBufSize - 1) >> 8) & 0xFF;
    uartDmaTxDescr->LENL       = (uartTxBufSize - 1) & 0xFF;
    uartDmaTxDescr->VLEN       = DMA_VLEN_FIXED;    // Use fixed length DMA transfer count.

    // Perform 1-byte transfers
    uartDmaTxDescr->WORDSIZE   = DMA_WORDSIZE_BYTE;

    // Transfer a single word after each DMA trigger
    uartDmaTxDescr->TMODE      = DMA_TMODE_SINGLE;

    // DMA word trigger = USARTx TX complete
    uartDmaTxDescr->TRIG       = DMA_TRIG_UTX0;

    uartDmaTxDescr->SRCINC     = DMA_SRCINC_1;          // Increment source pointer by 1 word
                                                        // address after each transfer.
    uartDmaTxDescr->DESTINC    = DMA_DESTINC_0;         // Do not increment destination pointer:
                                                        // points to USART UxDBUF register.
    uartDmaTxDescr->IRQMASK    = DMA_IRQMASK_ENABLE;    // Enable DMA interrupt to the CPU
    uartDmaTxDescr->M8         = DMA_M8_USE_8_BITS;     // Use all 8 bits for transfer count
    uartDmaTxDescr->PRIORITY   = DMA_PRI_LOW;           // DMA memory access has low priority

    // Link DMA descriptor with its corresponding DMA configuration register.
    if (uartDmaTxChan < 1)
    {
        DMA0CFGH = (uint8)((uint16)uartDmaTxDescr >> 8);
        DMA0CFGL = (uint8)((uint16)uartDmaTxDescr & 0x00FF);
    } else {
        DMA1CFGH = (uint8)((uint16)uartDmaTxDescr >> 8);
        DMA1CFGL = (uint8)((uint16)uartDmaTxDescr & 0x00FF);
    }

    // Arm the relevant DMA channel for UART TX, and apply 9 NOPs
    // to allow the DMA configuration to load
    DMAARM = (1 << uartDmaTxChan);
    NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP(); // 9 Nops

    DMAIF = 0;
    DMAIE = 1; 

    // Send the very first UART byte to trigger a UART TX session:
    U0DBUF = uartTxBuf[0];

    /*
    *   At this point the UART peripheral generates a DMA trigger each time it has
    *   transmitted a byte, leading to a DMA transfer from the allocated source buffer
    *   to the UxDBUF register. Once the DMA controller has completed the defined
    *   range of transfers, the CPU vectors its execution to the DMA ISR.
    */
}

/***********************************************************************************
* @fn          DMA_ISR
*
* @brief       DMA Interrupt Service Routine, called when DMA has finished
*              transferring one packet/buffer between memory and UxDBUF.
*
* @param       none
*
*/
#pragma vector = DMA_VECTOR
__interrupt void DMA_ISR(void)
{

    // Clear the main DMA interrupt Request Flag (IRCON.DMAIF = 0)
    DMAIF = 0;

    // Start a new UART RX session on DMA channel 1:
    if (DMAIRQ & DMAIRQ_DMAIF0)
    {
        // Indicate UART packet received (monitored by main-loop for data integrity/error check.
        //uartPktReceived = 1;

        // Clear DMA Channel 0 Interrupt Request Flag (DMAIRQ.DMAIF0 = 0).
        DMAIRQ = ~DMAIRQ_DMAIF0;

        // Re-arm DMA Channel 0 (DMAARM.DMAARM0 = 1).
        DMAARM |= DMAARM_DMAARM0;
    }

    // Start a new UART TX session on DMA channel 1:
    if (DMAIRQ & DMAIRQ_DMAIF1)
    {
		uartPktTransceived = 0;

        // Clear DMA Channel 1 Interrupt Request Flag (DMAIRQ.DMAIF1 = 0)
        DMAIRQ = ~DMAIRQ_DMAIF1;

        /*
        *   In this particular software example a new UART/DMA TX session must
        *   be initialized outside this DMA ISR. However, it would also be possible
        *   to use this DMA ISR to start a new UART/DMA TX session by simply
        *   re-arming the DMA channel and sending the first UART TX byte.
        */
    }
}
#endif
