/***********************************************************************************

Filename:     hal_rf.c

Description:

***********************************************************************************/

#include <ioCC2544.h>
#include <string.h>
#include "hal_rf.h"
#include "hal_mcu.h"
#include "prop_regs.h"


//***********************************************************************************
// Defines
// TX ouput power
#define CC2544_TXPOWER_4_DBM       0xF5
#define CC2544_TXPOWER_0_DBM       0xD5
#define CC2544_TXPOWER_MIN_6_DBM   0x95
#define CC2544_TXPOWER_MIN_23_DBM  0x05

// Basic RF mode
#define MODE_TX                     0
#define MODE_RX                     1

// Register to finetune Q phase relative I (i.e. skewness)
#define ACOMPQS           XREG(0x61B6)

//***********************************************************************************
// Variables
static uint16 frequency = 2400;



//***********************************************************************************
// Internal functions
static uint8 basicRf(uint8 mode, uint8 cw);

/***********************************************************************************
* @fn      halRfGetChipId
*
* @brief   Get chip id
*
* @param   none
*
* @return  uint8 - result
*/
uint8 halRfGetChipId(void)
{
  return CHIPID;
}

/***********************************************************************************
* @fn      halRfGetChipVer
*
* @brief   Get chip version
*
* @param   none
*
* @return  uint8 - result
*/
uint8 halRfGetChipVer(void)
{
  // return major revision (4 upper bits)
  return CHVER>>4;
}

/***********************************************************************************
** \brief	Initialize radio
*
* This will initialize the radio in raw mode
*
*/
void halRfInit(void)
{
  //-----------Configure modem--------------
  
  // Output power
  TXPOWER = CC2544_TXPOWER_4_DBM;
  
  // 2 Mbit/s mode - may change later!
  ADCTEST0 = 0x66;
  RXCTRL = 0x29;
  FSCTRL = 0x5A;
  LNAGAIN = 0x7F;
  
  //Set GFSK 2Mbps 320 kHz deviation
  MDMCTRL0 = 0x4E;
  // Set frequency offset average filter behaviour, and syncword correlation threshold
  MDMCTRL1 = 0x48;
  // Set DC offset estimation to before packet
  MDMTEST0 = 0xAF; // 128 samples average
  // Set Tx filter bandwidth (TBD)
  TXFILTCFG = 0x07;
  // Change DAC current step from '110' to '111'.
  TXCTRL = 0x79;
  // Enable AGC and override extra preamble bytes with tone and reduce synth calibration time
  PRF.PKT_CONF.AGC_EN = 1;
  PRF.PKT_CONF.START_TONE = 1;
  // Set number of preamble bytes to 13 (12+1). With the previous definition, the first 12 bytes will be overwritten with a tone.
  MDMCTRL2 = 0xCC;
  // Finetune Q phase relative I (i.e. skewness) 
  ACOMPQS = 0x16;
  
  // Clear radio memory
  memset((void*)RFCORE_RAM_PAGE, 0, RFCORE_RAM_PAGE_SZ);
  
  // No timer 2 events sent to LLE
  T2EVTCFG = 0x77;
  
  // Enable both whiteners to get pseudo-random data
  BSP_MODE = 0x03;
  
  //Enable link layer engine
  LLECTRL = 0x01;
}

/***********************************************************************************
** \brief	Set RF frequency
*
* Send RF frequency
*
* \param[in]       frequency
*     Frequency to set [2379-2495]
* @return  1 successful - 0 if fail
*/
uint8 halRfSetFrequency(uint16 newFrequency)
{
  // Check that valid RF frquency is requested
  if (newFrequency < 2379 || newFrequency > 2495){
    return 0;
  }
  frequency = newFrequency;
  
  return 1;
}

/***********************************************************************************
* @fn      halRfSetTxPower
*
* @brief   Set TX output power
*
* @param   uint8 power - power level: TXPOWER_4_DBM, TXPOWER_0_DBM,
*                        TXPOWER_MIN_6_DBM, TXPOWER_MIN_23_DBM
*
* @return  1 successful - 0 if fail
*/
uint8 halRfSetTxPower(uint8 power)
{
  uint8 n;
  
  switch(power)
  {
  case TXPOWER_4_DBM: n = CC2544_TXPOWER_4_DBM; break;
  case TXPOWER_0_DBM: n = CC2544_TXPOWER_0_DBM; break;
  case TXPOWER_MIN_6_DBM: n = CC2544_TXPOWER_MIN_6_DBM; break;
  case TXPOWER_MIN_23_DBM: n = CC2544_TXPOWER_MIN_23_DBM; break;
  
  default:
    return 0;
  }
  
  // Set TX power
  TXPOWER = n;
  
  return 1;
}


/***********************************************************************************
** \brief	Simple TX
*
* Simple TX test mode
*
* \param[in]       cw
*     CW on or off
* @return  1 successful - 0 if fail
*/
uint8 halRfSimpleTx(uint8 cw)
{
  return basicRf(MODE_TX, cw);
}

/***********************************************************************************
** \brief	Simple RX
*
* Simple RX test mode
*
* @return  1 successful - 0 if fail
*/
uint8 halRfSimpleRx(void)
{
  return basicRf(MODE_RX, CW_OFF);
}

/***********************************************************************************
** \brief	Simple RX RSSI update
*
* Simple RX test mode
*
* @return  RSSI value
*/
int8 halRfGetRssi(void)
{
  return RSSI - 96;
}

/***********************************************************************************
** \brief	Stop all operations
*
* Stop all RF operations
*
* @return  1 successful - 0 if fail
*/
uint8 halRfShutdown(void)
{
  RFST = 1;                       // Shutdown
  halMcuWaitMs(3);
  return (RFST == 0);
}

/***********************************************************************************
** \brief	Basic RF test
*
* Basic RF test mode
*
* \param[in]       mode
*     MODE_TX or MODE_RX
* \param[in]       cw
*     CW on or off
* @return  1 successful - 0 if fail
*/
static uint8 basicRf(uint8 mode, uint8 cw)
{
  // CW on and mode RX is not allowed
  if (cw == CW_ON && mode == MODE_RX){
    return 0;
  }
  
  if (cw){
    // Change mixer and PA current settings
    FSCTRL = 0x75;
    // Program frequency
    FREQCTRL = frequency - 2379;
    // Set modem to transmit tone
    MDMCTRL0 |= 0x20;
    // Transmitted tone has 0 Hz frequency
    MDMTEST1 = 0x0A;
    // Disable whiteners
    BSP_MODE = 0x00;
  }else{
    if (mode == MODE_TX) {
      // Change mixer and PA current settings
      FSCTRL = 0x75;
      // Program frequency - offset by +1 MHz or -1 MHz
      if (frequency < 2442) {
        // Use -1 MHz IF, so program frequency 1 MHz above carrier
        FREQCTRL = frequency - 2379 + 1;
        // Set -1 MHz IF
        MDMTEST1 = 0x25;
      }
      else {
        // Use +1 MHz IF, so program frequency 1 MHz below carrier
        FREQCTRL = frequency - 2379 - 1;
        // Set +1 MHz IF
        MDMTEST1 = 0x0F;
      }
    }
    else {
      // 0 IF for Rx
      FREQCTRL = frequency - 2379;
    }
    // Set modem to transmit modulated signal
    MDMCTRL0 &= ~0x20;
    // Enable and initialize both whiteners to have a long effective whitener
    BSP_MODE = 0x03;
    BSP_W = 0xFF;
  }
  
  if (mode == MODE_TX){
    RFST = 4; // CMD_TX_TEST
    if (RFERRF != 0){
      return 0;
    }
  } else {
    RFST = 3; // CMD_RX_TEST
    if (RFERRF != 0){
      return 0;
    }
  }
  return 1;
}

/*
+------------------------------------------------------------------------------
|  Copyright 2004-2007 Texas Instruments Incorporated. All rights reserved.
|
|  IMPORTANT: Your use of this Software is limited to those specific rights
|  granted under the terms of a software license agreement between the user who
|  downloaded the software, his/her employer (which must be your employer) and
|  Texas Instruments Incorporated (the "License"). You may not use this Software
|  unless you agree to abide by the terms of the License. The License limits
|  your use, and you acknowledge, that the Software may not be modified, copied
|  or distributed unless embedded on a Texas Instruments microcontroller or used
|  solely and exclusively in conjunction with a Texas Instruments radio
|  frequency transceiver, which is integrated into your product. Other than for
|  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
|  works of, modify, distribute, perform, display or sell this Software and/or
|  its documentation for any purpose.
|
|  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
|  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
|  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
|  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
|  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
|  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
|  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING
|  BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
|  CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
|  SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
|  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
|
|  Should you have any questions regarding your right to use this Software,
|  contact Texas Instruments Incorporated at www.TI.com.
|
+------------------------------------------------------------------------------
*/
