/***********************************************************************************

  Filename:	usb_hid.c

  Description:	Application support for the HID class

***********************************************************************************/

/***********************************************************************************
* INCLUDES
*/
#include "hal_int.h"
#include "hal_mcu.h"

#include "usb_firmware_library_headers.h"
#include "usb_hid_reports.h"
#include "usb_class_requests.h"
#include "usb_hid.h"

#include "ioCC2544.h"
#include "OEM_dbg.h"
#include "hal_rf_proprietary.h"
#include "OEM_Ver.h"
#include "usb_hid_cert.h"

/***********************************************************************************
* GLOBAL DATA
*/
KEYBOARD_IN_REPORT __data keyboard;
MOUSE_IN_REPORT __data mouse;
MOUSE_IN_BOOT_REPORT __data mouse_BootProtocol;
Joystick_IN_REPORT __data joystick;
uint8_t joystick_data_out;
extern __xdata char bootbyte @ 0x0001;
extern __xdata char bootbyte2 @ 0x0002;
extern __xdata char bootbyte3 @ 0x0003;
extern __xdata char bootbyte4 @ 0x0004;

uint8_t USB_OutBuffer[HID_CUSTOM_EPOUT_SIZE], USB_InBuffer[HID_CUSTOM_EPIN_SIZE];
uint8_t RF_StatusBuf[RF_STATUS_SIZE];
uint8_t armoury_data_out, armoury_data_out2;

uint8_t USB_others_InBuffer[HID_OTHER_EPIN_SIZE + 1];
uint8_t usb_others_data_out;

uint8_t ECMD_TimeoutCount;
uint8_t rf_check_ticker, fcc_duty_len = 0, fcc_hop_freq = 0;
RF_Flag_t rf_flag;
uint16_t factory_cmd_idx;

const char *FCC_Test_Log[10] = {
    "NoLog",
    "2403_CW",
    "2440_CW",
    "2480_CW",
    "2403_MOD",
    "2440_MOD",
    "2480_MOD",
    "2403_RxON",
    "2440_RxON",
    "2480_RxON",
};

extern void halEnterPowerMode(void);

void usbInProcess(void)
{
    uint8 oldEndpoint;

    // USB ready to accept new IN packet
    halIntOff();
    oldEndpoint = USBFW_GET_SELECTED_ENDPOINT();
    USBFW_SELECT_ENDPOINT(4);

    // The IN endpoint is ready to accept data
    if (USBFW_IN_ENDPOINT_DISARMED()) {
        // Write to USB FIFO
        usbfwWriteFifo(&USBF4, HID_CUSTOM_EPIN_SIZE, USB_InBuffer);
        USBFW_SELECT_ENDPOINT(4);
        USBFW_ARM_IN_ENDPOINT();   // Send data to the host
    }

    USBFW_SELECT_ENDPOINT(oldEndpoint);
    halIntOn();
}

void usbOutProcess(void)
{
    uint8 oldEndpoint;

    // If new packet is ready in USB FIFO
    halIntOff();
    oldEndpoint = USBFW_GET_SELECTED_ENDPOINT();
    USBFW_SELECT_ENDPOINT(4);

    if (USBFW_OUT_ENDPOINT_DISARMED()) {
        // Read from USB FIFO
        usbfwReadFifo(&USBF4, HID_CUSTOM_EPOUT_SIZE, USB_OutBuffer);
        USBFW_SELECT_ENDPOINT(4);
        USBFW_ARM_OUT_ENDPOINT();
    }
    USBFW_SELECT_ENDPOINT(oldEndpoint);
    halIntOn();
}

/** \brief Initializes the \ref module_usb_firmware_library_config module
*
* This function should be called first.
*/
void usbHidInit(void)
{
    // Init USB library
    usbfwInit();

    // Initialize the USB interrupt handler with bit mask containing all processed USBIRQ events
    usbirqInit(0xFFFF);

    // Enable pullup on D+
    HAL_USB_PULLUP_ENABLE();

    // Enable global interrupts
    halIntOn();

} // usbHidInit

/// @}

#define CMD_ENTRY_BOOTLOADER        0x7B

void usbHidProcessEvents(void)
{
    // Handle USB reset
    if (USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_RESET) {
        USBIRQ_CLEAR_EVENTS(0xFFFF);
        usbfwResetHandler();
        USBPOW = USBPOW_SUSPEND_EN;
    }

    // Handle USB packets on EP0
    if (USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_SETUP) {
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_SETUP);
        usbfwSetupHandler();
    }

    /// Handle USB packets on EP4
    #if 0 // for test
    if(USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_EP4OUT)
    {
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_EP4OUT);
        usbOutProcess(); // read USB out data from PC

        if((USB_OutBuffer[0] ==0x12) && (USB_OutBuffer[1] ==0x34))
        {
            USB_InBuffer[0] = 0x56;
            USB_InBuffer[1] = 0x78;
            usbInProcess();  // handle the USB out data
        }
    }
    #else
    if(USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_EP4OUT)
    {
        usbOutProcess(); // read USB out data from PC
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_EP4OUT);
    }
    #endif

    // Handle USB suspend
    if (USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_SUSPEND) {
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_SUSPEND);  // Clear USB suspend interrupt
        usbsuspEnter();                             // Calling this function will take the chip into PM1 until a USB resume is deteceted.
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_RESUME);   // Clear USB resume interrupt.
    }

    /* keybaord IN endpoint */
    if(USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_EP1IN)
    {
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_EP1IN);
    }

    /* mouse IN endpoint */
    if(USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_EP2IN)
    {
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_EP2IN);

    }

    /* custom HID IN endpoint */
    if(USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_EP4IN)
    {
        USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_EP4IN);

    }
}

void usbHidProcessKeyboard(uint8 *pRfData)
{
    uint8 i;

    keyboard.modifiers = pRfData[1];     // copy data from rx buffer.
    keyboard.reserved = pRfData[2];
    for(i = 0; i < sizeof(keyboard.pKeyCodes); i++) {
        keyboard.pKeyCodes[i] = pRfData[i + 3];
    }

    // Update and send the received HID keyboard report if USB endpoint is ready
    hidUpdateKeyboardInReport(&keyboard);

    if (hidSendKeyboardInReport()) {

        // For each successfully sent keyboard report, also clear the HID
        // keyboard report and send a blank report to release the key
        keyboard.modifiers = 0;
        keyboard.reserved = 0;
        for(i = 0; i < sizeof(keyboard.pKeyCodes); i++) {
            keyboard.pKeyCodes[i] = 0;
        }

        hidUpdateKeyboardInReport(&keyboard);
        for (i= 0;i<100; i++) {
            if (hidSendKeyboardInReport())
                break;
            halMcuWaitMs(1);
        }
    }
}

void usbHidProcessMouse(uint8 *pRfData)
{
    for (uint8 i = 0 ; i < sizeof(MOUSE_IN_REPORT) ; i++)
    {
        mouse.data[i] = *(pRfData + i);
    }

    // force disable key & wheel //
    //mouse.Key.Byte = 0;
    //mouse.Wheel    = 0;

    // Update and send the received HID mouse report if USB endpoint is ready
    hidUpdateMouseInReport(&mouse);
    hidSendMouseInReport();
}


/*
+------------------------------------------------------------------------------
|  Copyright 2004-2009 Texas Instruments Incorporated. All rights reserved.
|
|  IMPORTANT: Your use of this Software is limited to those specific rights
|  granted under the terms of a software license agreement between the user who
|  downloaded the software, his/her employer (which must be your employer) and
|  Texas Instruments Incorporated (the "License"). You may not use this Software
|  unless you agree to abide by the terms of the License. The License limits
|  your use, and you acknowledge, that the Software may not be modified, copied
|  or distributed unless embedded on a Texas Instruments microcontroller or used
|  solely and exclusively in conjunction with a Texas Instruments radio
|  frequency transceiver, which is integrated into your product. Other than for
|  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
|  works of, modify, distribute, perform, display or sell this Software and/or
|  its documentation for any purpose.
|
|  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
|  PROVIDED AS IS?WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
|  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
|  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
|  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
|  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
|  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING
|  BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
|  CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
|  SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
|  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
|
|  Should you have any questions regarding your right to use this Software,
|  contact Texas Instruments Incorporated at www.TI.com.
|
+------------------------------------------------------------------------------
*/





