/** \mainpage USB Firmware Library
 *
 * The Texas Instruments LPRF <b>USB Firmware Library</b> (Rev. B) supports:
 * \li Devices: CC1111, CC2511 and CC2531 RF USB Dongles
 * \li Tools:   IAR EW8051 7.51
 *
 * \section section_usblib_introduction Introduction
 * The USB Firmware Library is a software package which aids USB peripheral development with the
 * CC1111, CC2511 and CC2531 Low Power RF devices. It supports both vendor-specified device classes and standardized
 * device classes such as HID and USB audio, with any number of configurations, interfaces and alternate
 * settings. The following features are provided:
 * \li Initialization of the USB peripheral unit, and response to reset-signaling on the bus
 * \li Automated response to almost all standard requests, based on data from the USB descriptor set (the
 *     \ref SYNCH_FRAME and \ref SET_DESCRIPTOR requests must be handled by the application).
 * \li Automated setup of endpoint control registers, depending on the selected configuration and
 *     interface alternate settings.
 * \li All endpoint types are supported: Setup, bulk, interrupt and isochronous
 * \li An efficient interface for responding to class and vendor requests. Both automated and
 *     manual data transfer is supported
 * \li Endpoint access via a set of macros and functions
 * \li Support for suspend mode and remote wake-up
 * \li A flexible hook- and event-based interface allows for the framework to run from main, the USB
 *     interrupt, or a combination of both.
 *
 * The library has been designed to be generic and flexible. However, it does require that the application developer
 * has some knowledge knowledge of the USB specification, including the concepts of configurations, interfaces and
 * endpoints, different endpoint types, standard, vendor and class requests, and USB descriptor sets.
 *
 * \section section_usblib_usage Structure
 * The USB library is divided into five sub-modules:
 *
 * \image html usb_firmware_library_block_diagram.gif
 *
 * The \ref module_usb_framework contains all USB status information and functions for initialization,
 * device reset handling, and transfers on endpoint 0. It also contains basic macros and functions
 * for endpoint control and FIFO access. Vendor and class requests are passed up to the application, and
 * standard requests further down for internal processing.
 *
 * The \ref module_usb_standard_requests module contains automated processing functions for almost all
 * standard requests. The automated handling and endpoint configuration relies on the USB descriptor
 * set and two look-up tables. Requests that are not supported (\c SYNCH_FRAME and \c SET_DESCRIPTOR) or
 * that are class specific, are passed up to the application. The application is also notified upon
 * important events, such as change in endpoint status or interface alternate settings.
 *
 * The \ref module_usb_descriptor_parser module provides a mechanism for locating standard-formatted USB
 * descriptors. The user must provide a USB descriptor set, and two simple lookup tables - one for
 * locating other descriptor formats, and another for setting up endpoint double-buffering. The necessary
 * constant and structure definitions, and guidelines for writing compatible descriptor sets, are found in
 * the \ref module_usb_descriptor module.
 *
 * The \ref module_usb_suspend module provides easy to use, fully automated support for USB suspend,
 * resume and remote wakeup functionality. The only thing needed to implement suspend and resume is a
 * short piece of code in the main loop. Remote wakeup is performed by a single function call.
 *
 * The USB framework can run either from the main loop or the USB interrupt service routine.
 * The \ref module_usb_interrupt module converts all interrupts into events, and then calls the application.
 * The application can handle selected events immediately (in the interrupt), and leave the remaining for
 * later processing in the main loop.
 *
 * \section section_usblib_files Files
 * To support all code and data models and full debugging, the library is not provided as an already
 * compiled lib file. Instead the following files must be included in the project:
 *
 * Common Source Files:
 * - usb_descriptor_parser.c
 * - usb_framework.c
 * - usb_standard_requests.c
 *
 * CC1111/CC2511 Source Files
 * - ccxx11/usb_suspend.c
 * - ccxx11/usb_interrupt.c
 *
 * CC2531 Source Files
 * - cc2531/usb_suspend.c
 * - cc2531/usb_interrupt.c
 * - cc2531/hal_enter_pm.s51
 *
 * Header Files:
 * - usb_firmware_library_headers.h (includes the other header files listed below)
 * - usb_descriptor.h
 * - usb_descriptor_parser.h
 * - usb_reg.h
 * - usb_framework.h
 * - usb_framework_structs.h
 * - usb_interrupt.h
 * - usb_standard_requests.h
 * - usb_suspend.h
 *
 * \section section_usblib_getting_started Getting Started
 * It is highly recommended to have a close look at the applications examples provided with the USB Firmware Library
 * before starting development of a new application. These should serve as a guideline for implementing a USB  application
 * on the Texas Instruments LPRF USB dongles.
 *
 * The following steps are required to compile the application and enumerate the device on the PC:
 * - Write the \ref module_usb_descriptor set and the look-up tables, using the provided guidelines.
 *
 * - Provide the following hooks for vendor and class requests (refer to the \ref module_usb_framework
 *   module for further details):
 * \code
 * void usbcrHookProcessOut(void) 
 * {
 *    // Process USB class requests with OUT data phase, or stall endpoint 0 when unsupported
 * }
 * 
 * void usbcrHookProcessIn(void) 
 * {
 *    // Process USB class requests with IN data phase, or stall endpoint 0 when unsupported
 * }
 * 
 * void usbvrHookProcessOut(void) 
 * {
 *    // Process USB vendor requests with OUT data phase, or stall endpoint 0 when unsupported
 * }
 * 
 * void usbvrHookProcessIn(void) 
 * {
 *    // Process USB vendor requests with IN data phase, or stall endpoint 0 when unsupported
 * }
 * \endcode
 *
 * - Provide the following hooks for unsupported/unhandled standard requests (refer to the
 *   \ref module_usb_standard_requests module for further details):
 * \code
 * void usbsrHookSetDescriptor(void) 
 * {
 *    // Process SET_DESCRIPTOR requests, or stall endpoint 0 when unsupported
 * }
 * 
 * void usbsrHookSynchFrame(void) 
 * {
 *    // Process SYNCH_FRAME requests, or stall endpoint 0 when unsupported
 * }
 * 
 * void usbsrHookClearFeature(void) 
 * {
 *    // Process CLEAR_FEATURE requests that refer to non-standard features, or stall endpoint 0 when
 *    // unsupported
 * }
 * 
 * void usbsrHookSetFeature(void) 
 * {
 *    // Process SET_FEATURE requests that refer to non-standard features, or stall endpoint 0 when
 *    // unsupported
 * }
 * 
 * void usbsrHookModifyGetStatus(uint8 recipient, uint8 index, uint16 __xdata *pStatus) 
 * {
 *    // Modify *pStatus in a GET_STATUS request, or simply return if there are no class status flags
 * }
 * \endcode
 *
 * - Provide the following hook for standard request event processing (refer to the
 *   \ref module_usb_standard_requests module for further details):
 * \code
 * void usbsrHookProcessEvent(uint8 event, uint8 index) 
 * {
 *    // Process relevant events, one at a time.
 *    switch (event) {
 *    case USBSR_EVENT_INTERFACE_CHANGING: // <- Example
 *       ... Handle starting interface changes ...
 *       break;
 *    case ...
 *    }
 * }
 * \endcode
 *
 * - Provide the following hook for immediate interrupt event processing (refer to the
 *   \ref module_usb_interrupt module for further details):
 * \code
 * void usbirqHookProcessEvents(void)
 * {
 *    // Handle events that require immediate processing here
 *    // The same piece of code can be used to process low-priority events from main()
 *    if (USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_EP1IN) { // <- Example
 *       USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_EP1IN);
 *       ... Handle endpoint 1 events ...
 *    }
 * }
 * \endcode
 *
 * - Add the following initialization code to main():
 * \code
 * ... Initialize the 48 MHz crystal oscillator ...
 *
 * // Initialize the USB framework
 * usbfwInit();
 *
 * \endcode
 *
 * - Add the following code to the main loop or \ref usbirqHookProcessEvents():
 * \code
 * // Handle reset signaling on the bus
 * if (USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_RESET) {
 *    USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_RESET);
 *    usbfwResetHandler();
 * }
 *
 * // Handle USB setup transfers
 * if (USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_SETUP) {
 *    USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_SETUP);
 *    usbfwSetupHandler();
 * }
 * \endcode
 *
 * - Add the following code to the main loop and make sure that it runs at least every 10 ms. Do not run
 *   this code from any interrupt service routine (refer to the \ref module_usb_suspend module for
 *   further details):
 * \code
 * // Handle suspend and resume signalling on the bus
 * if (USBIRQ_GET_EVENT_MASK() & USBIRQ_EVENT_SUSPEND) {
 *    USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_SUSPEND);
 *
 *    ... Do what needs to be done before entering power mode 1 here (e.g. turn off the radio, configure
 *        I/O to minimize power consumption, start the sleep timer etc.) ...
 *
 *    // The usbsuspEnter() function will return after resume (initiated by the host) or remote wakeup
 *    // (initiated by the device)
 *    usbsuspEnter();
 *    USBIRQ_CLEAR_EVENTS(USBIRQ_EVENT_RESUME);
 *
 *    ... If a USBIRQ_EVENT_RESET event will affect the code that follows (before the event is processed
 *        elsewhere), then make sure to handle it here also ...
 *
 *    ... Do what needs to be done to wake up from suspend mode (e.g. turn on the radio, reactivate I/O
 *        and peripherals, turn off the sleep timer ...
 * }
 * \endcode
 * @{
 * @}
 */
