/*
*	OEM_Service.C
*
*   OEM service related functions.
*
*	Author: Hugo Lin - Asus
*	email: Hugo_Lin@asus.com
*
*/

#include <stdlib.h>
#include <xdc/std.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Error.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Event.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/knl/Semaphore.h>
#include <ti/sysbios/knl/Idle.h>
#include <ti/drivers/timer/GPTimerCC26XX.h>
#include <icall.h>

/* sensor control header files */
/*
#include "ex_include_tirtos.h"
#include "scif.h"
#include "scif_framework.h"
#define BV(n)               (1 << (n))
*/
/* Board Header files */
#include "Board.h"
#include "OEM_Include.h"

#define MAIN_SERVICE_STACK_SIZE         640
#define MAIN_TASK_PRIORITY              4
#define MAIN_TASK_FOR_BLE_PRIORITY      1

extern ICall_SyncHandle bleAppEvent;
extern Event_Handle rf_event_handle;

static Task_Struct mainServiceTask;
static uint8_t mainServiceTaskStack[MAIN_SERVICE_STACK_SIZE];
Event_Handle serviceEvent;
Clock_Handle service_clock_1ms;
Clock_Struct standby_clock;
BITS_32 system_flag;
DWORD device_mode;
bool ble_wait_flag = FALSE;
uint8_t ble_wait_cnt = 0;
GPTimerCC26XX_Handle hTimer1;

static void mainServiceTaskFunction(UArg arg0, UArg arg1);

Void clockHandler1(UArg arg)
{
    if (Is_BLE_Mode_Only()) {
        #define BLE_POLLING_TIME 8
        if (++ble_wait_cnt >= BLE_POLLING_TIME) {
            ble_wait_flag = TRUE;
            ble_wait_cnt = 0;
        }
    }
    Event_post(serviceEvent, OEM_SERVICE_1MS_EVENT);
}

//#pragma optimize=none
void hw_delay_5us(uint32_t Delay_Time)
{
    GPTimerCC26XX_start(hTimer1);
    //Max delay time:(273)1.365ms
    HWREG( GPT1_BASE + GPT_O_TAV ) = 0;
    while( HWREGH( GPT1_BASE + GPT_O_TAV ) < ( 240 * Delay_Time ) - 1 );
    GPTimerCC26XX_stop(hTimer1);
}

#pragma optimize=none
void sw_delay_1us(uint32_t Delay_Time)
{
    while (Delay_Time) {
        for (int i = 5; i > 0; i--) {};
        Delay_Time--;
    }
}

void start_oem_service_clock(void)
{
    Clock_start(service_clock_1ms);
}

void stop_oem_service_clock(void)
{
    Clock_stop(service_clock_1ms);
}

static void mainServiceTaskFunction(UArg arg0, UArg arg1)
{
    Error_Block  eb;
    UInt         events;
    Clock_Params clockParams;

    Error_init(&eb);

    serviceEvent = Event_create(NULL, &eb);
    //if (serviceEvent == NULL) {
    //    System_abort("event create failed");
    //}

    Clock_Params_init(&clockParams);
    clockParams.period = 100;
    clockParams.startFlag = TRUE; /* start immediately */
    service_clock_1ms = Clock_create((Clock_FuncPtr)clockHandler1, 100, &clockParams, &eb);
    //if (service_clock_1ms == NULL) {
    //    System_abort("oem service clock create failed");
    //}

    check_uart_need_start(); //force enable UART
    mouse_key_interrupt_enable();

    while(1) {
        events = Event_pend(serviceEvent, Event_Id_NONE, OEM_SERVICE_WAIT_EVENT, BIOS_WAIT_FOREVER);
        //DBG_byte_MAIN(events);
        //DBG_puts_MAIN("=event");
        if (events & OEM_SERVICE_1MS_EVENT) {
            service_1ms();
        }

        if (events & OEM_SERVICE_UART_EVENT) {
            check_uart_need_start();
        }
    }
}

void MainService_createTask(void)
{
    //uint8 *mainServiceTaskStack = ICall_malloc(MAIN_SERVICE_STACK_SIZE);
    Task_Params mainServiceTaskParams;
    Task_Params_init( &mainServiceTaskParams );
    mainServiceTaskParams.stackSize = MAIN_SERVICE_STACK_SIZE;
    if (Is_USB_RF_Mode()) {
        mainServiceTaskParams.priority = MAIN_TASK_PRIORITY;
    } else {
        mainServiceTaskParams.priority = MAIN_TASK_FOR_BLE_PRIORITY;
    }
    mainServiceTaskParams.stack = mainServiceTaskStack;

    Task_construct(&mainServiceTask, mainServiceTaskFunction, &mainServiceTaskParams, NULL);
}

static void powerOffServiceTaskFunction(UArg arg0, UArg arg1)
{
    Error_Block  eb;

    Error_init(&eb);
    serviceEvent = Event_create(NULL, &eb);

    GPTImer_Initial();
    //sensor_enter_shutdown();
    GPTimer_DeInit();

    OEM_UART_DeInit();
    OEM_SPI_Deinit(Is_USB_Mode());
    OEM_SPI_Enter_Sleep_Mode(Is_USB_Mode());
    OEM_TRNG_DeInit();
    Out_RF_Boot_Inactive(); // turn off RF_BOOT when optic sensor is idle.

    while (1) {
        Event_pend(serviceEvent, Event_Id_NONE, 0x00, BIOS_WAIT_FOREVER);
    }
}

void PowerOffService_createTask(void)
{
    //uint8 *mainServiceTaskStack = ICall_malloc(MAIN_SERVICE_STACK_SIZE);
    Task_Params mainServiceTaskParams;
    Task_Params_init(&mainServiceTaskParams);
    mainServiceTaskParams.stackSize = MAIN_SERVICE_STACK_SIZE;
    mainServiceTaskParams.priority = MAIN_TASK_PRIORITY;
    mainServiceTaskParams.stack = mainServiceTaskStack;

    Task_construct(&mainServiceTask, powerOffServiceTaskFunction, &mainServiceTaskParams, NULL);
}

void GPTImer_Initial(void)
{
    GPTimerCC26XX_Params GPT1params;

    GPTimerCC26XX_Params_init(&GPT1params);
    GPT1params.width          = GPT_CONFIG_16BIT;
    GPT1params.mode           = GPT_MODE_PERIODIC_UP;
    GPT1params.debugStallMode = GPTimerCC26XX_DEBUG_STALL_OFF;

    hTimer1 = GPTimerCC26XX_open(CC2640R2RC_GPTIMER1A, &GPT1params);
    if (hTimer1 == NULL) {
        //Log_error0("Failed to open GPTimer");
        //Task_exit();
    }

    GPTimerCC26XX_Value loadVal = 65536;
    GPTimerCC26XX_setLoadValue(hTimer1, loadVal);
}

void GPTimer_DeInit(void)
{
    GPTimerCC26XX_close(hTimer1);
}

void service_1ms(void)
{
    //Do not change the sequence - Start
    if (uart_tx_need_check) {
        uart_tx_need_check = 0;
        check_uart_need_start();
    }
    //Do not change the sequence - End
}



