/******************************************************************************

 @file       hid_adv_remote.c

 @brief HID Advanced Remote Application.

 Group: CMCU, SCS
 Target Device: CC2640R2

 ******************************************************************************

 Copyright (c) 2015-2017, Texas Instruments Incorporated
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 *  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

 *  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

 *  Neither the name of Texas Instruments Incorporated nor the names of
    its contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 ******************************************************************************

 *****************************************************************************/

/******************************************************************************
 * INCLUDES
 */
#include <xdc/std.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/knl/Event.h>
#include <ti/sysbios/knl/Queue.h>
#include <ti/sysbios/hal/Hwi.h>

#include <icall.h>
#include <string.h>
#include "util.h"
#include "att_rsp.h"

/* This Header file contains all BLE API and icall structure definition */
#include "icall_ble_api.h"
#include "devinfoservice.h"
#include "battservice.h"
//#include "hidkbdccservice.h"
#include "hidkbmservice.h"
#include "hiddev.h"
#include "peripheral.h"
#include "Board.h"
#include "ll_common.h"
#include "smartrf_settings/smartrf_settings.h"
#include "OEM_Include.h"
#ifdef USE_RCOSC
#include "rcosc_calibration.h"
#endif //USE_RCOSC
//#include "OEM_Armoury_BLE.h"


/******************************************************************************
 * MACROS
 */

/******************************************************************************
 * CONSTANTS
 */

// Advertising interval when device is discoverable (units of 625us, 160=100ms)
#define DEFAULT_ADVERTISING_INTERVAL          48

// HID idle timeout in msec; set to zero to disable timeout
#define DEFAULT_HID_IDLE_TIMEOUT              180000

// Minimum connection interval (units of 1.25ms, 80=100ms) if automatic
// parameter update request is enabled
#define DEFAULT_DESIRED_MIN_CONN_INTERVAL     6

// Maximum connection interval (units of 1.25ms, 800=1000ms) if automatic
// parameter update request is enabled
#define DEFAULT_DESIRED_MAX_CONN_INTERVAL     12

// Slave latency to use if automatic parameter update request is enabled
#define DEFAULT_DESIRED_SLAVE_LATENCY         30

// Supervision timeout value (units of 10ms, 1000=10s) if automatic
// parameter update request is enabled
#define DEFAULT_DESIRED_CONN_TIMEOUT          200

// Whether to enable automatic parameter update request when a connection is
// formed
#define DEFAULT_ENABLE_UPDATE_REQUEST         GAPROLE_LINK_PARAM_UPDATE_WAIT_REMOTE_PARAMS

// Connection Pause Peripheral time value (in seconds)
#define DEFAULT_CONN_PAUSE_PERIPHERAL         2

// Length of MITM passcode
#define PASSCODE_LEN                          6

// Default GAP pairing mode
#define DEFAULT_PAIRING_MODE                  GAPBOND_PAIRING_MODE_INITIATE

// Default MITM mode (TRUE to require passcode or OOB when pairing)
#define DEFAULT_MITM_MODE                     FALSE

// Default bonding mode, TRUE to bond
#define DEFAULT_BONDING_MODE                  TRUE

// Default GAP bonding I/O capabilities
#define DEFAULT_IO_CAPABILITIES               GAPBOND_IO_CAP_NO_INPUT_NO_OUTPUT  // GAPBOND_IO_CAP_KEYBOARD_ONLY

// Battery level is critical when it is less than this %
#define DEFAULT_BATT_CRITICAL_LEVEL           26

// Task configuration
#define BLE_TASK_PRIORITY                     2

#ifndef BLE_TASK_STACK_SIZE
#define BLE_TASK_STACK_SIZE                   1024
#endif

// Application specific event ID for HCI Connection Event End Events
#define SBP_HCI_CONN_EVT_END_EVT              0x0001

// Application events
#define SBP_STATE_CHANGE_EVT                  0x0001
#define SBP_CHAR_CHANGE_EVT                   0x0002
#define SBP_PAIRING_STATE_EVT                 0x0004
#define SBP_CONN_EVT                          0x0010

// Set initial values to maximum, RX is set to max. by default(251 octets, 2120us)
#define APP_SUGGESTED_PDU_SIZE                251   // default is 27 octets(TX)
#define APP_SUGGESTED_TX_TIME                 2120  // default is 328us(TX)
/******************************************************************************
 * TYPEDEFS
 */
typedef struct
{
  appEvtHdr_t hdr;  // event header.
  uint8_t *pData;  // event data
} sbpEvt_t;

/******************************************************************************
 * GLOBAL VARIABLES
 */

/******************************************************************************
 * EXTERNAL VARIABLES
 */
extern bool HidDev_bonding_status;

/******************************************************************************
 * EXTERNAL FUNCTIONS
 */

/******************************************************************************
 * LOCAL VARIABLES
 */

// Entity ID globally used to check for source and/or destination of messages
static ICall_EntityID selfEntity;

// Event globally used to post local events and pend on system and
// local events.
ICall_SyncHandle bleAppEvent;

// Task configuration
Task_Struct bleTask;
Char bleTaskStack[BLE_TASK_STACK_SIZE];
#ifdef CUSTOM_PROFILE_ENABLE
static Queue_Struct appMsg;
static Queue_Handle appMsgQueue;
#endif
Mouse_ReportData_t mouse_report_data;
uint8_t KB_Report_Data[HID_KEYBOARD_IN_RPT_LEN];

// GAP - SCAN RSP data (max size = 31 bytes)
static uint8_t scanRspData[] =
{
  BLE_SCAN_RSP_LEN,                             // length of this data
  GAP_ADTYPE_LOCAL_NAME_COMPLETE,   // AD Type = Complete local name
  BLE_DEVICE_NAME
};

// Advertising data
static uint8_t advertData[] =
{
  // flags
  0x02,   // length of this data
  GAP_ADTYPE_FLAGS,
  GAP_ADTYPE_FLAGS_LIMITED | GAP_ADTYPE_FLAGS_BREDR_NOT_SUPPORTED,

  // appearance
  0x03,   // length of this data
  GAP_ADTYPE_APPEARANCE,
  LO_UINT16(GAP_APPEARE_HID_MOUSE),
  HI_UINT16(GAP_APPEARE_HID_MOUSE),
#ifdef CUSTOM_PROFILE_ENABLE
  // service UUIDs
  0x07,   // length of this data
  GAP_ADTYPE_16BIT_MORE,
  LO_UINT16(HID_SERV_UUID),
  HI_UINT16(HID_SERV_UUID),
  LO_UINT16(BATT_SERV_UUID),
  HI_UINT16(BATT_SERV_UUID),
  LO_UINT16(ARMOURYPROFILE_SERV_UUID),
  HI_UINT16(ARMOURYPROFILE_SERV_UUID),
#else
  // service UUIDs
  0x05,   // length of this data
  GAP_ADTYPE_16BIT_MORE,
  LO_UINT16(HID_SERV_UUID),
  HI_UINT16(HID_SERV_UUID),
  LO_UINT16(BATT_SERV_UUID),
  HI_UINT16(BATT_SERV_UUID),
#endif

#ifdef SWIFT_PAIR_ENABLE
  // Swift Pair for windows 10 RS4(1803) above
  BLE_ADVERT_DEVICE_NAME_LEN,     //Length
  0xff,     //Vendor Defined Flag
  0x06,     //MicrosoftVendor ID
  0x00,     //MicrosoftVendor ID
  0x03,     //MicrosoftBeacon ID
  0x00,     //MicrosoftBeacon Scenario
  0x80,     //Reserved RSSIByte
  BLE_DEVICE_NAME
#endif
};

// Device name attribute value
static uint8_t attDeviceName[] = BLE_DEVICE_NAME_STR;
static const uint16 appearance = GAP_APPEARE_HID_MOUSE;

// HID Dev configuration
static hidDevCfg_t hidAdvRemoteCfg =
{
  DEFAULT_HID_IDLE_TIMEOUT,   // Idle timeout
  HID_KBD_FLAGS               // HID feature flags
};

#define HID_MOUSE_IN_RPT_LEN    sizeof(Mouse_ReportData_t)

// Application GAP Role and GAP Bond states
static gaprole_States_t bleGapRoleState = GAPROLE_INIT;
static uint8_t bleGapBondState = HID_GAPBOND_PAIRING_STATE_NONE;

/******************************************************************************
 * LOCAL FUNCTIONS
 */

static void HIDAdvRemote_init(void);
static void HIDAdvRemote_taskFxn(UArg a0, UArg a1);

static void HIDAdvRemote_processStackMsg(ICall_Hdr *pMsg);
static uint8_t HIDAdvRemote_processGattMsg(gattMsgEvent_t *pMsg);

static uint8_t HIDAdvRemote_handleReportCB(uint8_t id, uint8_t type,
                                           uint16_t uuid, uint8_t oper,
                                           uint16_t *pLen, uint8_t *pData);
static void HIDAdvRemote_handleEventCB(uint8_t evt);
#ifdef CUSTOM_PROFILE_ENABLE
static uint8_t ArmouryBLEPeripheral_enqueueMsg(uint8_t event, uint8_t state,
                                              uint8_t *pData);
static void ArmouryBLEPeripheral_connEvtCB(Gap_ConnEventRpt_t *pReport);
static void ArmouryBLEPeripheral_processCharValueChangeEvt(uint8_t paramID);
static void ArmouryBLEPeripheral_processAppMsg(sbpEvt_t *pMsg);
static void ArmouryBLEPeripheral_charValueChangeCB(uint8_t paramID);
#endif
static void Close_Advetising(void);
static void Enable_Advetising(void);

/******************************************************************************
 * PROFILE CALLBACKS
 */
// HID Dev callbacks
static hidDevCB_t hidAdvRemoteHidCBs =
{
    HIDAdvRemote_handleReportCB,
    HIDAdvRemote_handleEventCB,
};

// GAP Bond Manager Callbacks
// These are set to NULL since they are not needed. The application
// is set up to only perform justworks pairing.
// Armoury GATT Profile Callbacks (not define)
#ifdef CUSTOM_PROFILE_ENABLE
static ArmouryProfileCBs_t ArmouryBLEPeripheral_ArmouryProfileCBs =
{
  ArmouryBLEPeripheral_charValueChangeCB // Armoury GATT Characteristic value change callback
};
#endif
/******************************************************************************
 * PUBLIC FUNCTIONS
 */
/*********************************************************************
 * The following typedef and global handle the registration to connection event
 */
typedef enum
{
   NOT_REGISTER       = 0,
   FOR_AOA_SCAN       = 1,
   FOR_ATT_RSP        = 2,
   FOR_AOA_SEND       = 4,
   FOR_TOF_SEND       = 8
}connectionEventRegisterCause_u;

// Set the register cause to the registration bit-mask
#define CONNECTION_EVENT_REGISTER_BIT_SET(RegisterCause) (connectionEventRegisterCauseBitMap |= RegisterCause )
// Remove the register cause from the registration bit-mask
#define CONNECTION_EVENT_REGISTER_BIT_REMOVE(RegisterCause) (connectionEventRegisterCauseBitMap &= (~RegisterCause) )
// Gets whether the current App is registered to the receive connection events
#define CONNECTION_EVENT_IS_REGISTERED (connectionEventRegisterCauseBitMap > 0)
// Gets whether the RegisterCause was registered to recieve connection event
#define CONNECTION_EVENT_REGISTRATION_CAUSE(RegisterCause) (connectionEventRegisterCauseBitMap & RegisterCause )

// Handle the registration and un-registration for the connection event, since only one can be registered.
uint32_t       connectionEventRegisterCauseBitMap = NOT_REGISTER; //see connectionEventRegisterCause_u
bStatus_t HIDAdvRemote_RegistertToAllConnectionEvent (connectionEventRegisterCause_u connectionEventRegisterCause);

/*********************************************************************
 * @fn      HIDAdvRemote_createTask
 *
 * @brief   Task creation function for the HID Advanced Remote.
 *
 * @param   None.
 *
 * @return  None.
 */
void HIDAdvRemote_createTask(void)
{
    Task_Params taskParams;

    // Configure task
    Task_Params_init(&taskParams);
    taskParams.stack = bleTaskStack;
    taskParams.stackSize = BLE_TASK_STACK_SIZE;
    taskParams.priority = BLE_TASK_PRIORITY;

    Task_construct(&bleTask, HIDAdvRemote_taskFxn, &taskParams, NULL);
}

#ifdef SWIFT_PAIR_ENABLE
void change_advertising_data_without_swift_pair(void)
{
    GAPRole_SetParameter(GAPROLE_ADVERT_DATA, sizeof(advertData) - (BLE_ADVERT_DEVICE_NAME_LEN + 1), advertData);
}
#endif

/*********************************************************************
 * @fn      HIDAdvRemote_init
 *
 * @brief   Initialization function for the Advance Remote App Task.
 *          This is called during initialization and should contain
 *          any application specific initialization (ie. hardware
 *          initialization/setup, table initialization, power up
 *          notification ...).
 *
 * @param   None.
 *
 * @return  None.
 */
static void HIDAdvRemote_init(void)
{
    // ******************************************************************
    // N0 STACK API CALLS CAN OCCUR BEFORE THIS CALL TO ICall_registerApp
    // ******************************************************************
    // Register the current thread as an ICall dispatcher application
    // so that the application can send and receive messages.
    ICall_registerApp(&selfEntity, &bleAppEvent);

#ifdef USE_RCOSC
    RCOSC_enableCalibration();
#endif // USE_RCOSC

#ifdef CUSTOM_PROFILE_ENABLE
    appMsgQueue = Util_constructQueue(&appMsg);
#endif

    HCI_EXT_SetTxPowerCmd(HCI_EXT_TX_POWER_4_DBM);

    // Disable Data Length Extension Feature
    HCI_EXT_SetMaxDataLenCmd(APP_SUGGESTED_PDU_SIZE, APP_SUGGESTED_TX_TIME, APP_SUGGESTED_PDU_SIZE, APP_SUGGESTED_TX_TIME);

    // Setup the GAP
    VOID GAP_SetParamValue(TGAP_CONN_PAUSE_PERIPHERAL, DEFAULT_CONN_PAUSE_PERIPHERAL);

    // Setup the GAP Peripheral Role Profile
    {
        // For all hardware platforms, device starts advertising upon initialization
        uint8_t initial_advertising_enable = false;

//        if (!(system_flag.fbit.factory_mode && FCC_Type)) {
            if (HidDev_bondCount()) {
                HidDev_bonding_status = true;
                initial_advertising_enable = false;
            } else {
                initial_advertising_enable = true;
//                Pairing_Mode_Init();
            }
//        }
        // By setting this to zero, the device will go into the waiting state after
        // being discoverable for 30.72 second, and will not being advertising again
        // until the enabler is set back to TRUE
        uint16_t gapRole_AdvertOffTime = 0;   // 0 seconds

        uint8_t enable_update_request = DEFAULT_ENABLE_UPDATE_REQUEST;
        uint16_t desired_min_interval = DEFAULT_DESIRED_MIN_CONN_INTERVAL;
        uint16_t desired_max_interval = DEFAULT_DESIRED_MAX_CONN_INTERVAL;
        uint16_t desired_slave_latency = DEFAULT_DESIRED_SLAVE_LATENCY;
        uint16_t desired_conn_timeout = DEFAULT_DESIRED_CONN_TIMEOUT;

        // Set the GAP Role Parameters
        GAPRole_SetParameter(GAPROLE_ADVERT_ENABLED, sizeof(uint8_t), &initial_advertising_enable);
        GAPRole_SetParameter(GAPROLE_ADVERT_OFF_TIME, sizeof(uint16_t), &gapRole_AdvertOffTime);
#ifdef SWIFT_PAIR_ENABLE
        if (HidDev_bonding_status) {
            change_advertising_data_without_swift_pair();
        } else {
            GAPRole_SetParameter(GAPROLE_ADVERT_DATA, sizeof(advertData), advertData);
        }
#else
        GAPRole_SetParameter(GAPROLE_ADVERT_DATA, sizeof(advertData), advertData);
#endif
        GAPRole_SetParameter(GAPROLE_SCAN_RSP_DATA, sizeof(scanRspData), scanRspData);
        GAPRole_SetParameter(GAPROLE_PARAM_UPDATE_ENABLE, sizeof(uint8_t), &enable_update_request);
        GAPRole_SetParameter(GAPROLE_MIN_CONN_INTERVAL, sizeof(uint16_t), &desired_min_interval);
        GAPRole_SetParameter(GAPROLE_MAX_CONN_INTERVAL, sizeof(uint16_t), &desired_max_interval);
        GAPRole_SetParameter(GAPROLE_SLAVE_LATENCY, sizeof(uint16_t), &desired_slave_latency);
        GAPRole_SetParameter(GAPROLE_TIMEOUT_MULTIPLIER, sizeof(uint16_t), &desired_conn_timeout);
    }

    // Set the GAP Characteristics
    {
        GGS_SetParameter(GGS_DEVICE_NAME_ATT, GAP_DEVICE_NAME_LEN, attDeviceName);
        GGS_SetParameter(GGS_APPEARANCE_ATT, sizeof(uint16), (void *)&appearance );
    }

    // Set advertising interval
    {
        uint16_t advInt = DEFAULT_ADVERTISING_INTERVAL;

        GAP_SetParamValue(TGAP_LIM_DISC_ADV_INT_MIN, advInt);
        GAP_SetParamValue(TGAP_LIM_DISC_ADV_INT_MAX, advInt);
        GAP_SetParamValue(TGAP_GEN_DISC_ADV_INT_MIN, advInt);
        GAP_SetParamValue(TGAP_GEN_DISC_ADV_INT_MAX, advInt);

        GAP_SetParamValue(TGAP_LIM_ADV_TIMEOUT, BLE_ADV_TIMEOUT);//setup advertising
    }

    // Setup the GAP Bond Manager
    {
        uint8_t pairMode           = DEFAULT_PAIRING_MODE;
        uint8_t mitm               = DEFAULT_MITM_MODE;
        uint8_t ioCap              = DEFAULT_IO_CAPABILITIES;
        uint8_t bonding            = DEFAULT_BONDING_MODE;
        uint8_t scMode             = GAPBOND_SECURE_CONNECTION_NONE;
        //uint8_t removeWhenListFull = TRUE;

        GAPBondMgr_SetParameter(GAPBOND_PAIRING_MODE, sizeof(uint8_t), &pairMode);
        GAPBondMgr_SetParameter(GAPBOND_MITM_PROTECTION, sizeof(uint8_t), &mitm);
        GAPBondMgr_SetParameter(GAPBOND_IO_CAPABILITIES, sizeof(uint8_t), &ioCap);
        GAPBondMgr_SetParameter(GAPBOND_BONDING_ENABLED, sizeof(uint8_t), &bonding);
        GAPBondMgr_SetParameter(GAPBOND_SECURE_CONNECTION, sizeof(uint8_t), &scMode);
        //GAPBondMgr_SetParameter( GAPBOND_LRU_BOND_REPLACEMENT, sizeof(uint8_t), &removeWhenListFull );
    }

    // Setup Battery Characteristic Values
    {
        uint8_t critical = DEFAULT_BATT_CRITICAL_LEVEL;

        Batt_SetParameter(BATT_PARAM_CRITICAL_LEVEL, sizeof(uint8_t), &critical);
    }

    // Add HID keyboard service
    HidKbM_AddService();

#ifdef CUSTOM_PROFILE_ENABLE
    ArmouryProfile_AddService(GATT_ALL_SERVICES);

    // Register callback with ArmouryGATTprofile
    ArmouryProfile_RegisterAppCBs(&ArmouryBLEPeripheral_ArmouryProfileCBs);
#endif
    // Register for HID Dev callback
    HidDev_Register(&hidAdvRemoteCfg, &hidAdvRemoteHidCBs);

    // Start the GAP Role and Register the Bond Manager.
    HidDev_StartDevice();

    // Register with GAP for HCI/Host messages
    GAP_RegisterForMsgs(selfEntity);

#ifdef CUSTOM_PROFILE_ENABLE
    // Register for GATT local events and ATT Responses pending for transmission
    GATT_RegisterForMsgs(selfEntity);
#endif
    //Set default values for Data Length Extension
    {
        //This API is documented in hci.h
        //See the LE Data Length Extension section in the BLE-Stack User's Guide for information on using this command:
        //http://software-dl.ti.com/lprf/sdg-latest/html/cc2640/index.html
        HCI_LE_WriteSuggestedDefaultDataLenCmd(APP_SUGGESTED_PDU_SIZE, APP_SUGGESTED_TX_TIME);
    }

#if !defined (USE_LL_CONN_PARAM_UPDATE)
    // Get the currently set local supported LE features
    // The HCI will generate an HCI event that will get received in the main
    // loop
    HCI_LE_ReadLocalSupportedFeaturesCmd();
#endif // !defined (USE_LL_CONN_PARAM_UPDATE)

//    if (!(system_flag.fbit.factory_mode && FCC_Type)) {
        HidDev_StartAdvertising();
//    }
}

/*********************************************************************
 * @fn      HIDADvRemote_taskFxn(BLE Task)
 *
 * @brief   Application task entry point for the HID Advance Remote Application.
 *
 * @param   a0, a1 - not used.
 *
 * @return  None.
 */
static void HIDAdvRemote_taskFxn(UArg a0, UArg a1)
{
    HIDAdvRemote_init();

    while(1) {
        uint32_t localEvents;

        localEvents = Event_pend(bleAppEvent, Event_Id_NONE, BLE_ALL_EVENTS, ICALL_TIMEOUT_FOREVER);

        if (localEvents) {
            ICall_EntityID dest;
            ICall_ServiceEnum src;
            ICall_HciExtEvt *pMsg = NULL;

            if (ICall_fetchServiceMsg(&src, &dest, (void **)&pMsg) == ICALL_ERRNO_SUCCESS) {
#ifdef CUSTOM_PROFILE_ENABLE
                ICall_Stack_Event *pEvt = (ICall_Stack_Event *)pMsg;
                 // Check for BLE stack events first
                if (pEvt->signature == 0xffff) {
                    // The GATT server might have returned a blePending as it was trying
                    // to process an ATT Response. Now that we finished with this
                    // connection event, let's try sending any remaining ATT Responses
                    // on the next connection event.
                    if (pEvt->event_flag & SBP_HCI_CONN_EVT_END_EVT) {
                        // Try to retransmit pending ATT Response (if any)
                        //SimpleBLEPeripheral_sendAttRsp();
                    }
                }
#endif
                if ((src == ICALL_SERVICE_CLASS_BLE) && (dest == selfEntity)) {
                    // Process inter-task message
                    HIDAdvRemote_processStackMsg((ICall_Hdr *) pMsg);
                }

                if (pMsg) {
                    ICall_freeMsg(pMsg);
                }
            }
        }

#ifdef CUSTOM_PROFILE_ENABLE
        if (localEvents & BLE_QUEUE_EVT) {
            while (!Queue_empty(appMsgQueue)) {
                sbpEvt_t *pMsg = (sbpEvt_t *)Util_dequeueMsg(appMsgQueue);
                if (pMsg) {
                    // Process message.
                    ArmouryBLEPeripheral_processAppMsg(pMsg);
                    // Free the space from the message.
                    ICall_free(pMsg);
                }
            }
        }
#endif

        if (localEvents & BLE_HID_MS_REPORT) {
            HidDev_Report(HID_RPT_ID_MOUSE_IN,
                          HID_REPORT_TYPE_INPUT,
                          HID_MOUSE_IN_RPT_LEN,
                          (uint8_t*)&mouse_report_data);

#ifdef CUSTOM_PROFILE_ENABLE
            Armoury_Ack_Notification();
#endif
        }

        if (localEvents & BLE_HID_KB_REPORT) {
            HidDev_Report(HID_RPT_ID_KEY_IN,
                          HID_REPORT_TYPE_INPUT,
                          HID_KEYBOARD_IN_RPT_LEN,
                          KB_Report_Data);
        }

        if (localEvents & BLE_PAIRING_EVT) {
            if (HidDev_bonding_status) {
                HidDev_bonding_status = false;
                BLE_Start_Advertising();
            } else {
                Enable_Advetising();
            }
        }

        if (localEvents & BLE_SLEEP_EVT) {
            Close_Advetising();
            stop_ble_clock();
        }

        if (localEvents & BLE_WAKEUP_EVT) {
            start_ble_clock();
        }

        if (localEvents & BLE_STOP_PAIRING_EVT) {
            GAPRole_TerminateConnection();
        }

        if (localEvents & BLE_CHECK_PARAM_EVT) {
#if 1
            uint16 interval;
            GAPRole_GetParameter(GAPROLE_CONN_INTERVAL, &interval);
            if (interval > DEFAULT_DESIRED_MIN_CONN_INTERVAL && interval <= DEFAULT_DESIRED_MAX_CONN_INTERVAL) {
                GAPRole_SendUpdateParam(DEFAULT_DESIRED_MIN_CONN_INTERVAL,
                                        DEFAULT_DESIRED_MIN_CONN_INTERVAL,
                                        DEFAULT_DESIRED_SLAVE_LATENCY,
                                        DEFAULT_DESIRED_CONN_TIMEOUT,
                                        GAPROLE_RESEND_PARAM_UPDATE);
            }
#endif
        }
    }
}

/*********************************************************************
 * @fn      HIDAdvRemote_processStackMsg
 *
 * @brief   Process an incoming stack message.
 *
 * @param   pMsg - message to process
 *
 * @return  TRUE if safe to deallocate incoming message, FALSE otherwise.
 */
static void HIDAdvRemote_processStackMsg(ICall_Hdr *pMsg)
{
    switch (pMsg->event)
    {
        case GATT_MSG_EVENT:
            // Process GATT message
            HIDAdvRemote_processGattMsg((gattMsgEvent_t *)pMsg);
        break;

        case HCI_GAP_EVENT_EVENT:
        {

            // Process HCI message
            switch(pMsg->status)
            {
                case HCI_COMMAND_COMPLETE_EVENT_CODE:
                // Process HCI Command Complete Event
                {
#if !defined (USE_LL_CONN_PARAM_UPDATE)
                    // This code will disable the use of the LL_CONNECTION_PARAM_REQ
                    // control procedure (for connection parameter updates, the
                    // L2CAP Connection Parameter Update procedure will be used
                    // instead). To re-enable the LL_CONNECTION_PARAM_REQ control
                    // procedures, define the symbol USE_LL_CONN_PARAM_UPDATE
                    // The L2CAP Connection Parameter Update procedure is used to
                    // support a delta between the minimum and maximum connection
                    // intervals required by some iOS devices.

                    // Parse Command Complete Event for opcode and status
                    hciEvt_CmdComplete_t* command_complete = (hciEvt_CmdComplete_t*) pMsg;
                    uint8_t   pktStatus = command_complete->pReturnParam[0];

                    //find which command this command complete is for
                    switch (command_complete->cmdOpcode)
                    {
                        case HCI_LE_READ_LOCAL_SUPPORTED_FEATURES:
                        {
                            if (pktStatus == SUCCESS)
                            {
                                uint8_t featSet[8];

                                // Get current feature set from received event (bits 1-9
                                // of the returned data
                                memcpy( featSet, &command_complete->pReturnParam[1], 8 );

                                // Clear bit 1 of byte 0 of feature set to disable LL
                                // Connection Parameter Updates
                                CLR_FEATURE_FLAG( featSet[0], LL_FEATURE_CONN_PARAMS_REQ );

                                // Update controller with modified features
                                HCI_EXT_SetLocalSupportedFeaturesCmd( featSet );
                            }
                        }
                        break;

                        default:
                        //do nothing
                        break;
                    }
#endif // !defined (USE_LL_CONN_PARAM_UPDATE)
                }
                break;

                default:
                break;
            }
        }
        break;

        default:
        // do nothing
        break;
    }
}

/*********************************************************************
 * @fn      HIDAdvRemote_processGattMsg
 *
 * @brief   Process GATT messages
 *
 * @param   pMsg - message to process
 *
 * @return  None.
 */
static uint8_t HIDAdvRemote_processGattMsg(gattMsgEvent_t *pMsg)
{
#ifdef CUSTOM_PROFILE_ENABLE
    uint16 currentMTUSize;

    // See if GATT server was unable to transmit an ATT response
    if (attRsp_isAttRsp(pMsg))
    {
        // No HCI buffer was available. Let's try to retransmit the response
        // on the next connection event.
        if( HIDAdvRemote_RegistertToAllConnectionEvent(FOR_ATT_RSP) == SUCCESS)
        {
            // Don't free the response message yet
            return (FALSE);
        }
    }
    else if (pMsg->method == ATT_FLOW_CTRL_VIOLATED_EVENT)
    {
        // ATT request-response or indication-confirmation flow control is
        // violated. All subsequent ATT requests or indications will be dropped.
        // The app is informed in case it wants to drop the connection.

        // Display the opcode of the message that caused the violation.
        //Display_print1(dispHandle, 5, 0, "FC Violated: %d", pMsg->msg.flowCtrlEvt.opcode);
    }
    else if (pMsg->method == ATT_MTU_UPDATED_EVENT)
    {
        // MTU size updated
        currentMTUSize = pMsg->msg.mtuEvt.MTU;
        //SDITask_setAppDataSize(currentMTUSize);
        DBG_byte_BLE(currentMTUSize);
        DBG_puts_BLE("=MTU");
    }
#endif
    // Free message payload. Needed only for ATT Protocol messages
    GATT_bm_free(&pMsg->msg, pMsg->method);

    // It's safe to free the incoming message
    return (TRUE);
}

/*********************************************************************
 * @fn      HIDAdvRemote_handleReportCB
 *
 * @brief   HID Dev report callback.
 *
 * @param   id    - HID report ID.
 * @param   type  - HID report type.
 * @param   uuid  - attribute uuid.
 * @param   oper  - operation:  read, write, etc.
 * @param   pLen  - Length of report.
 * @param   pData - Report data.
 *
 * @return  SUCCESS
 */
static uint8_t HIDAdvRemote_handleReportCB(uint8_t id, uint8_t type,
                                           uint16_t uuid, uint8_t oper,
                                           uint16_t *pLen, uint8_t *pData)
{
    // Intentionally left blank
    return SUCCESS;
}

/*********************************************************************
 * @fn      HIDAdvRemote_handleEventCB
 *
 * @brief   HID Dev event callback.
 *
 * @param   evt - event ID.
 *
 * @return  None.
 */
static void HIDAdvRemote_handleEventCB(uint8_t evt)
{
    switch(evt)
    {
        case(HID_DEV_GAPROLE_STATE_CHANGE_EVT):
            // Update application GAP Role state
            HidDev_GetParameter(HIDDEV_GAPROLE_STATE, &bleGapRoleState);
        break;

        case(HID_DEV_GAPBOND_STATE_CHANGE_EVT):
            // Update application GAP Bond pairing state
            HidDev_GetParameter(HIDDEV_GAPBOND_STATE, &bleGapBondState);
        break;

        default:
            // Do nothing
        break;
    }

    return;
}

#ifdef CUSTOM_PROFILE_ENABLE
/*********************************************************************
 * @fn      ArmouryBLEPeripheral_charValueChangeCB
 *
 * @brief   Callback from Armoury Profile indicating a characteristic
 *          value change.
 *
 * @param   paramID - parameter ID of the value that was changed.
 *
 */
bStatus_t HIDAdvRemote_RegistertToAllConnectionEvent (connectionEventRegisterCause_u connectionEventRegisterCause)
{
  bStatus_t status = SUCCESS;

  // in case  there is no registration for the connection event, make the registration
  if (!CONNECTION_EVENT_IS_REGISTERED)
  {
    status = GAP_RegisterConnEventCb(ArmouryBLEPeripheral_connEvtCB, GAP_CB_REGISTER, LINKDB_CONNHANDLE_ALL);
  }
  if(status == SUCCESS)
  {
    //add the reason bit to the bitamap.
    CONNECTION_EVENT_REGISTER_BIT_SET(connectionEventRegisterCause);
  }

  return(status);
}

/*********************************************************************
 * @fn      SimplePeripheral_connEvtCB
 *
 * @brief   Connection event callback.
 *
 * @param pReport pointer to connection event report
 */
static void ArmouryBLEPeripheral_connEvtCB(Gap_ConnEventRpt_t *pReport)
{
  // Enqueue the event for processing in the app context.
  if( ArmouryBLEPeripheral_enqueueMsg(SBP_CONN_EVT, 0 ,(uint8_t *) pReport) == FALSE)
  {
    ICall_free(pReport);
  }
}

#ifdef CUSTOM_PROFILE_ENABLE
static void ArmouryBLEPeripheral_charValueChangeCB(uint8_t paramID)
{
  ArmouryBLEPeripheral_enqueueMsg(SBP_CHAR_CHANGE_EVT, paramID, 0);
}
#endif

/*********************************************************************
 * @fn      ArmouryBLEPeripheral_enqueueMsg
 *
 * @brief   Creates a message and puts the message in RTOS queue.
 *
 * @param   event - message event.
 * @param   state - message state.
 * @param   pData - message data pointer.
 *
 * @return  TRUE or FALSE
 */
static uint8_t ArmouryBLEPeripheral_enqueueMsg(uint8_t event, uint8_t state,
                                           uint8_t *pData)
{
    sbpEvt_t *pMsg = ICall_malloc(sizeof(sbpEvt_t));

    // Create dynamic pointer to message.
    if (pMsg) {
        pMsg->hdr.event = event;
        pMsg->hdr.state = state;
        pMsg->pData = pData;

        // Enqueue the message.
        return Util_enqueueMsg(appMsgQueue, bleAppEvent, (uint8_t *)pMsg);
    }

    return FALSE;
}

/*********************************************************************
 * @fn      ArmouryBLEPeripheral_processAppMsg
 *
 * @brief   Process an incoming callback from a profile.
 *
 * @param   pMsg - message to process
 *
 * @return  None.
 */
#ifdef CUSTOM_PROFILE_ENABLE
static void ArmouryBLEPeripheral_processAppMsg(sbpEvt_t *pMsg)
{
  switch (pMsg->hdr.event)
  {
    case SBP_CHAR_CHANGE_EVT:
      {
        ArmouryBLEPeripheral_processCharValueChangeEvt(pMsg->hdr.state);
      }
      break;

    default:
      // Do nothing.
      break;
  }
}
#endif

/*********************************************************************
 * @fn      ArmouryBLEPeripheral_processCharValueChangeEvt
 *
 * @brief   Process a pending Simple Profile characteristic value change
 *          event.
 *
 * @param   paramID - parameter ID of the value that was changed.
 *
 * @return  None.
 */
static void ArmouryBLEPeripheral_processCharValueChangeEvt(uint8_t paramID)
{
  uint8_t newValue;

  switch(paramID)
  {
    case ARMOURYPROFILE_CHAR1:
      ArmouryProfile_GetParameter(ARMOURYPROFILE_CHAR1, &newValue);

      break;

    default:
      // should not reach here!
      break;
  }
}
#endif

void BLE_Start_Advertising(void)
{
    uint8_t advertising_enable;
    uint8_t advEventType = GAP_ADTYPE_ADV_IND;
    uint16_t advInt = DEFAULT_ADVERTISING_INTERVAL;
    uint8_t param = GAP_FILTER_POLICY_ALL;;

    GAPRole_TerminateConnection();

    advertising_enable = false;
    VOID GAPRole_SetParameter(GAPROLE_ADVERT_ENABLED, sizeof(uint8_t), &advertising_enable);
#ifdef SWIFT_PAIR_ENABLE
    VOID GAPRole_SetParameter(GAPROLE_ADVERT_DATA, sizeof(advertData), advertData);
#endif

    VOID HidDev_SetParameter(HIDDEV_ERASE_ALLBONDS, 0, NULL);
    HidDev_Set_reportIsNotReady();
    //BLE_Clear_BindTable();
    OEM_BLE_NVID_Reset();

    VOID GAP_SetParamValue(TGAP_LIM_DISC_ADV_INT_MIN, advInt);
    VOID GAP_SetParamValue(TGAP_LIM_DISC_ADV_INT_MAX, advInt);
    VOID GAP_SetParamValue(TGAP_GEN_DISC_ADV_INT_MIN, advInt);
    VOID GAP_SetParamValue(TGAP_GEN_DISC_ADV_INT_MAX, advInt);
    VOID GAP_SetParamValue(TGAP_LIM_ADV_TIMEOUT, BLE_ADV_TIMEOUT);

    VOID GAPRole_SetParameter(GAPROLE_ADV_FILTER_POLICY, sizeof(uint8_t), &param);
    VOID GAPRole_SetParameter(GAPROLE_ADV_EVENT_TYPE, sizeof(uint8), &advEventType);

    advertising_enable = true;
    VOID GAPRole_SetParameter(GAPROLE_ADVERT_ENABLED, sizeof(uint8_t), &advertising_enable);
    DBG_puts_BLE("S_Adv");
}

void access_ti_via_ble(uint16_t len, uint8_t* pData)
{
    static uint8 key_backup;
    uint8 key_change = 0;
    MouseData_t mouse_info;
#if 0
    if (len > MOUSE_DATA_SIZE && pData[8] == ICMD_KEYBOARD) {
        memcpy(KB_Report_Data, &pData[9], HID_KEYBOARD_IN_RPT_LEN);
        Event_post(bleAppEvent, BLE_HID_KB_REPORT);
    }

    memcpy(&mouse_info.data[0], pData, MOUSE_DATA_SIZE);
    if (key_backup != mouse_info.ReportD.Key.Byte) {
        key_change = 1;
        key_backup = mouse_info.ReportD.Key.Byte;
    }

    if (mouse_info.ReportD.X || mouse_info.ReportD.Y || mouse_info.ReportD.Wheel || key_change) {
        mouse_report_data.X = mouse_info.ReportD.X;
        mouse_report_data.Y = mouse_info.ReportD.Y;
        mouse_report_data.Wheel = mouse_info.ReportD.Wheel;
        mouse_report_data.Key.Byte = mouse_info.ReportD.Key.Byte;
        Event_post(bleAppEvent, BLE_HID_MS_REPORT);
    }
#endif
}

void stop_ble_clock(void)
{
    HidDev_StopIdleTimer();
}

void start_ble_clock(void)
{
    HidDev_StartIdleTimer();
}

void Set_BLE_IdleTimeOut(uint16_t TimeOut)
{
    hidAdvRemoteCfg.idleTimeout = TimeOut * 100;
}

static void Close_Advetising(void)
{
    uint8_t advertising_enable = false;

    GAPRole_SetParameter(GAPROLE_ADVERT_ENABLED, sizeof(uint8_t), &advertising_enable);
}

static void Enable_Advetising(void)
{
    uint8_t advertising_enable = true;

    GAPRole_SetParameter(GAPROLE_ADVERT_ENABLED, sizeof(uint8_t), &advertising_enable);
}


